//
//************************************************************************
// File                 : dlgpalet.cpp  
// Module               : 	
// Project              : View dumpfile  
// Description          :
// Creation date        : 23-Apr-1995 
// Last update          : 
// Author(s)            : Bertho Stultiens (BS)
// Version              :
// Compiler             : Borland C++ 3.1
//	switches	:
// Remarks              :
// Modification History :
//************************************************************************
//

#include <owl.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <dir.h>
#include <dos.h>
#include <bwcc.h>

#include "viewdmpr.h"
#include "dlgpalet.h"
#include "ini.h"


//
//*****************************************************************************
// Function	: TPaletteDialog::TPaletteDialog
// Syntax	: TPaletteDialog(PTWindowsObject aParent, LPSTR aTitle, TPaletteFile **PalFile, PTModule aModule)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
TPaletteDialog::TPaletteDialog(PTWindowsObject aParent, LPSTR aTitle, TPaletteFile **PalFile, PTModule aModule)
		: TDialog(aParent, aTitle, aModule)
{
	char path[MAXPATH];
	char drive[MAXDRIVE];
	char spath[MAXPATH];
	char lpath[MAXPATH];
	struct ffblk ffblk;
	int done;
	int cnt;
	PTPaletteFile pal;

	HCURSOR OldC = SetCursor(LoadCursor(NULL, IDC_WAIT));

        // Initialize vars
        Initialized	= FALSE;
	PaletteList	= NULL;
	PaletteLast	= NULL;
	hPalette	= NULL;
	LogPalette	= NULL;
	BBitmapInfo	= NULL;
        ParentPalette	= PalFile;


	spath[0] = path[0] = '\0';

        // Create a path to the palette files
	if(!GetIniParameter(eip_PaletteDir, spath, sizeof(spath)))
	{
        	// If no ini-file entry look in executable's directory for palette files
		GetModuleFileName(GetApplication()->hInstance, spath, sizeof(spath));
		fnsplit(spath, drive, lpath, NULL, NULL);
		fnmerge(spath, drive, lpath, "*", ".pal");
		fnmerge(path, drive, lpath, NULL, NULL);
		strcat(path, "/"); 
	}
	else
	{
		strcpy(path, spath);
		strcat(path, "/");
                strcat(spath, "/*.pal");
	}

	// Create a list of palette files (*.pal)
	done = findfirst(spath, &ffblk, 0);
	while (!done) 
	{
		if((ffblk.ff_attrib & (FA_DIREC | FA_LABEL | FA_SYSTEM)) == 0)
		{
                	// Only keep real files (archive, readonly and/or hidden)
			strcpy(lpath, path);
        	        strcat(lpath, ffblk.ff_name);
			if((pal = LoadPaletteFile(lpath)) != NULL)
			{
				if(PaletteLast == NULL)
				{
                        		PaletteList = PaletteLast = pal;
				}
				else
				{
					PaletteLast->Next = pal;
                                	PaletteLast = pal;
	                        }
			}
                }
		done = findnext(&ffblk);
	}

	// Allocate bitmap headers and set color indices
	if((BBitmapInfo = (BITMAPINFO *)malloc(sizeof(BITMAPINFO) + 256*sizeof(PALETTEENTRY))) == NULL)
	{
		Error("Cannot create Button Bitmapheader");
                SetCursor(OldC);
		return;
	}
	BBitmapInfo->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
	BBitmapInfo->bmiHeader.biWidth		= 4;
	BBitmapInfo->bmiHeader.biHeight		= MAXCOLORS;
	BBitmapInfo->bmiHeader.biPlanes		= 1;
	BBitmapInfo->bmiHeader.biBitCount	= 8;
	BBitmapInfo->bmiHeader.biCompression	= BI_RGB;
	BBitmapInfo->bmiHeader.biSizeImage	= BBitmapInfo->bmiHeader.biWidth * BBitmapInfo->bmiHeader.biHeight;
	BBitmapInfo->bmiHeader.biXPelsPerMeter	= 0;
	BBitmapInfo->bmiHeader.biYPelsPerMeter	= 0;
	BBitmapInfo->bmiHeader.biClrUsed	= 0;
	BBitmapInfo->bmiHeader.biClrImportant	= 0;

	for(cnt = 0; cnt < 256; cnt++)
	{
		((int *)&(BBitmapInfo->bmiColors))[cnt] = cnt;
	}
										
	// Fill the Button bitmap bits
	for(cnt = 10; cnt < MAXCOLORS+10; cnt++)
	{
		BBits[cnt-10] = (long)cnt + ((long)cnt<<8) + ((long)cnt<<16) + ((long)cnt<<24);
	}

	// Allocate a logical palette
	if((LogPalette = (LOGPALETTE *)malloc(sizeof(LOGPALETTE) + 256*sizeof(PALETTEENTRY))) == NULL)
	{
        	Error("Cannot create LogPalette");
		SetCursor(OldC);
		return;
	}
	LogPalette->palVersion = 0x300;
	LogPalette->palNumEntries = 256;
	HDC DC = GetDC(NULL);
	GetSystemPaletteEntries(DC, 0, 10, &LogPalette->palPalEntry[0]);
	GetSystemPaletteEntries(DC, 246, 10, &LogPalette->palPalEntry[246]);
	ReleaseDC(NULL, DC);

        // When arrived here, we can continue
        Initialized = TRUE;
	SetCursor(OldC);
}


//
//*****************************************************************************
// Function	: TPaletteDialog::~TPaletteDialog
// Syntax	: ~TPaletteDialog()
// Input	:
// Output	: 
// Description	:
// Remarks	:
//*****************************************************************************
//
TPaletteDialog::~TPaletteDialog()
{
	PTPaletteFile tmp;

	while(PaletteList != NULL)
	{
		tmp = PaletteList;
                PaletteList = PaletteList->Next;
                delete tmp;
	}

	if(BBitmapInfo != NULL)	free(BBitmapInfo);
        if(LogPalette != NULL)	free(LogPalette);
	if(hPalette != NULL)	DeleteObject(hPalette);
}


//
//*****************************************************************************
// Function	: TPaletteDialog::SetupWindow
// Syntax	: void SetupWindow(void)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TPaletteDialog::SetupWindow(void)
{
	TDialog::SetupWindow();

	if(!Initialized)
	{
		CloseWindow();
                return;
	}

	PTPaletteFile pal = PaletteList;
	int index = 0;
	char path[MAXPATH];

	while(pal != NULL)
        {
		SendDlgItemMsg(ID_PALLISTBOX, LB_ADDSTRING, 0, (LONG)(pal->Name));
                pal = pal->Next;
        }

	// Get path of last selected palette and selection index needed in listbox
	if(*ParentPalette == NULL)
        {
		if(GetIniParameter(eip_LastPaletteLoaded, path, sizeof(path)))
		{
			if((pal = LoadPaletteFile(path)) != NULL)
			{
				index = (int)SendDlgItemMsg(ID_PALLISTBOX, LB_FINDSTRINGEXACT,(WPARAM)0,(LPARAM)(pal->Name));
				delete pal;
			}
		}
	}
	else
	{
		index = (int)SendDlgItemMsg(ID_PALLISTBOX, LB_FINDSTRINGEXACT,(WPARAM)0,(LPARAM)((*ParentPalette)->Name));
	}

	if(index == LB_ERR) index = 0;

	// Select last selected palette
	SendDlgItemMsg(ID_PALLISTBOX, LB_SETCURSEL, (WPARAM)index, 0L);
	int len = (int)SendDlgItemMsg(ID_PALLISTBOX, LB_GETTEXTLEN, (WPARAM)index, 0L);
	char *name = new char[len+1];
	SendDlgItemMsg(ID_PALLISTBOX, LB_GETTEXT, (WPARAM)index, (LPARAM)name);

	if((CurrentPalette = FindPaletteName(name)) == NULL)
	{
		Error("Cannot locate current selected palette");
                CloseWindow();
                return;
	}
	else
	{
		memcpy(&LogPalette->palPalEntry[10], CurrentPalette->Palette, MAXCOLORS*sizeof(PALETTEENTRY));
        }
	delete name;

        // Copy curent palette
	if((hPalette = CreatePalette(LogPalette)) == NULL)
	{
               	Error("Cannot create Palette");
		return;
	}
}

void TPaletteDialog::IDPalListbox(RTMessage Msg)
{
	int index;

	if(Msg.LP.Hi == LBN_SELCHANGE)
	{
		index = (int)SendDlgItemMsg(ID_PALLISTBOX, LB_GETCURSEL, 0, 0L);
		int len = (int)SendDlgItemMsg(ID_PALLISTBOX, LB_GETTEXTLEN, (WPARAM)index, 0L);
		char *name = new char[len+1];
		SendDlgItemMsg(ID_PALLISTBOX, LB_GETTEXT, (WPARAM)index, (LPARAM)name);

		if((CurrentPalette = FindPaletteName(name)) == NULL)
		{
			Error("Cannot locate current selected palette");
                	CloseWindow();
        	        return;
		}
		else
		{
			memcpy(&LogPalette->palPalEntry[10], CurrentPalette->Palette, MAXCOLORS*sizeof(PALETTEENTRY));
			AnimatePalette(hPalette, 10, MAXCOLORS, CurrentPalette->Palette);
			InvalidateRect(GetItemHandle(ID_PALBUTTON), NULL, FALSE);
	        }
		delete name;
	}
}

void TPaletteDialog::DrawItem(DRAWITEMSTRUCT _FAR & di)
{
	// this dialogbox has only one item that has the ownerdraw flag
	// so the type and state info of the item in the message are not used

	UnrealizeObject(hPalette);	// To be sure to get the colors mapped
	int bltmode = SetStretchBltMode(di.hDC, STRETCH_DELETESCANS);	// Eliminate non fitting pixels
	HPALETTE hp = SelectPalette(di.hDC, hPalette, FALSE);
	RealizePalette(di.hDC);
	StretchDIBits(di.hDC,
		      di.rcItem.left, di.rcItem.top,
		      di.rcItem.right-di.rcItem.left, di.rcItem.bottom-di.rcItem.top,
		      0, 0,
		      sizeof(BBits[0]), MAXCOLORS,
		      BBits, BBitmapInfo, DIB_PAL_COLORS, SRCCOPY);
        SelectPalette(di.hDC, hp, FALSE);
	SetStretchBltMode(di.hDC, bltmode);
}


void TPaletteDialog::Ok(RTMessage Msg)
{
	PTPaletteFile pal = new TPaletteFile(CurrentPalette);

	if(pal != NULL && !pal->Initialized)
	{
		delete pal;
		TDialog::Cancel(Msg);
	}
	else if(pal == NULL)
	{
		TDialog::Cancel(Msg);
	}
	else
	{
		*ParentPalette = pal;
        }

	TDialog::Ok(Msg);
}


void TPaletteDialog::Cancel(RTMessage Msg)
{
	TDialog::Cancel(Msg);
}


void TPaletteDialog::WMQueryNewPalette(RTMessage Msg)
{

	HDC DC = GetDC(HWindow);
	HPALETTE opal = SelectPalette(DC, hPalette, FALSE);
	int cnt = RealizePalette(DC);
	SelectPalette(DC, opal, FALSE);
	ReleaseDC(HWindow, DC);
	if(cnt > 0)
	{
		InvalidateRect(HWindow, NULL, TRUE);
	        Msg.Result = 1L;
	}
	else
	{
        	Msg.Result = 0L;
        }

}


//
//*****************************************************************************
// Function	: TPaletteDialog::LoadPaletteFile
// Syntax	: PTPaletteFile LoadPaletteFile(char *path)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
PTPaletteFile TPaletteDialog::LoadPaletteFile(char *path)
{
	PTPaletteFile pal = new TPaletteFile(path);

	if(pal != NULL && pal->Initialized)
	{
        	return pal;
	}
	else
	{
        	if(pal != NULL) delete pal;
        	return NULL;
        }
}


//
//*****************************************************************************
// Function	: TPaletteDialog::FindPaletteName
// Syntax	: PTPaletteFile FindPaletteName(char *name)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
PTPaletteFile TPaletteDialog::FindPaletteName(char *name)
{
	PTPaletteFile pal = PaletteList;

	while(pal != NULL)
	{
		if(!strcmp(name, pal->Name))
		{
                	break;
		}
                pal = pal->Next;
        }
	return pal;
}


//
//*****************************************************************************
// Function	: TPaletteDialog::Error
// Syntax	: void Error(char *fmt, ...)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TPaletteDialog::Error(char *fmt, ...)
{
	char str[256];
	va_list va;

	va_start(va, fmt);
	vsprintf(str, fmt, va);
	BWCCMessageBox(HWindow, str, "Error...", MB_OK | MB_ICONHAND);
	va_end(va);
}


//
//*****************************************************************************
// Function	: TPaletteDialog::Question
// Syntax	: int Question(char *fmt, ...)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
int TPaletteDialog::Question(char *fmt, ...)
{
	char str[256];
	va_list va;
	int retval;

	va_start(va, fmt);
	vsprintf(str, fmt, va);
	retval = BWCCMessageBox(HWindow, str, "Question...", MB_YESNO | MB_ICONQUESTION);
	va_end(va);
        return retval;
}


