//
//************************************************************************
// File                 : dumpwin.cpp  
// Module               : 	
// Project              : View Dumpfile  
// Description          :
// Creation date        : 
// Last update          : 
// Author(s)            : Bertho Stultiens (BS)
// Version              :
// Compiler             : Borland C++ 3.1
//	switches	:
// Remarks              :
// Modification History :
//************************************************************************
// 

#include <owl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <dir.h>
#include <bwcc.h>
#include "dlgparm.h"
#include "viewdmp.h"
#include "dumpwin.h"
#include "dlgpalet.h"
#include "palfile.h"
#include "ini.h"

#define WINWMIN	200
#define WINHMIN	300

//
//*****************************************************************************
// Function	: TDumpWindow::TDumpWindow
// Syntax	: TDumpWindow(PTWindowsObject AParent, LPSTR ATitle,
//			      LPSTR AFileName, PTModule AModule)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
TDumpWindow::TDumpWindow(PTWindowsObject AParent, LPSTR ATitle, LPSTR AFileName, PTModule AModule)
	     : TWindow(AParent, ATitle, AModule)
{
	HCURSOR OldC = SetCursor(LoadCursor(NULL, IDC_WAIT));

	int cnt;
        RECT cr;

	// Put the window in a position
        GetClientRect(AParent->HWindow, &cr);
	int xsize = cr.right - cr.left;
	int ysize = cr.bottom - cr.top;
	Attr.W = xsize * 9 / 10;
	Attr.H = ysize * 9 / 10;
//	Attr.X = abs((xsize - Attr.W)/2);
//	Attr.Y = abs((ysize - Attr.H)/2);

	// Be sure these are initialized for early out returns (destructor checks)
        Initialized 	= FALSE;
	hArray		= NULL;
	hLArray		= NULL;
	hOrgBits	= NULL;
	hZoomBits	= NULL;
	PArray		= NULL;
	PLArray		= NULL;
	POrgBits	= NULL;
	PZoomBits	= NULL; 
	Array		= NULL;
	LArray		= NULL;
	OrgBits		= NULL;
	ZoomBits	= NULL;
	BitmapInfo	= NULL;
	ZBitmapInfo	= NULL;
	CSBitmapInfo	= NULL;
        hPalette	= NULL;
	LogPalette	= NULL;
        PaletteFile	= NULL;
	hFont		= NULL;
	csidx		= -1;

	ActionState = eAct_Nop;
	IsZoomedFlag = FALSE;

	ZoomT = 0;
	ZoomB = YSIZE-1;
	ZoomL = 0;
	ZoomR = XSIZE-1;

	Cursor		= LoadCursor(NULL, IDC_ARROW);
        CursorZoom	= LoadCursor(GetApplication()->hInstance, "CURSOR_ZOOM");
	CursorSetCenter	= LoadCursor(GetApplication()->hInstance, "CURSOR_SETCENTER");

	// Allocate the needed memory
	if((hArray = GlobalAlloc(GHND, lXSIZE * lYSIZE * sizeof(double))) == NULL)
	{
		Error("No memory for main array");
		SetCursor(OldC);
		return;
	}
			
	if((hLArray = GlobalAlloc(GHND, lXSIZE * lYSIZE * sizeof(long))) == NULL)
	{
		Error("No memory for long array");
		SetCursor(OldC);
		return;
	}

		// Note the +1l in the size. The GDI gives a GPF with some display drivers.
		// It is caused by an extra read beyond the array (if on a segment boundary).  	
	if((hOrgBits = GlobalAlloc(GHND, lXSIZE * lYSIZE + 1l)) == NULL)
	{
		Error("No memory for org bitmap bits");
		SetCursor(OldC);
		return;
	}
		
	if((hZoomBits = GlobalAlloc(GHND, lXSIZE * lYSIZE + 1l)) == NULL)
	{
		Error("No memory for zoom bitmap bits");
		SetCursor(OldC);
		return;
	}
		
	if((Array = (double **)malloc(YSIZE * sizeof(double *))) == NULL)
	{
		Error("No memory for main array access");
		SetCursor(OldC);
		return;
	}
		
	if((LArray = (long **)malloc(YSIZE * sizeof(long *))) == NULL)
	{
		Error("No memory for long array access");
		SetCursor(OldC);
		return;
	}

	if((OrgBits = (char **)malloc(YSIZE * sizeof(char *))) == NULL)
	{
		Error("No memory for org bitmap access");
		SetCursor(OldC);
		return;
	}

	if((ZoomBits = (char **)malloc(YSIZE * sizeof(char *))) == NULL)
	{
		Error("No memory for zoom bitmap access");
		SetCursor(OldC);
		return;
	}

	// Lock memory
	if((PArray = (double huge *)GlobalLock(hArray)) == NULL)
	{
		Error("Cannot lock Array");
		SetCursor(OldC);
		return;
	}	

	if((PLArray = (long huge *)GlobalLock(hLArray)) == NULL)
	{
		Error("Cannot lock long Array");
		SetCursor(OldC);
		return;
	}	

	if((POrgBits = (char huge *)GlobalLock(hOrgBits)) == NULL)
	{
		Error("Cannot lock org bits");
		SetCursor(OldC);
		return;
	}	

	if((PZoomBits = (char huge *)GlobalLock(hZoomBits)) == NULL)
	{
		Error("Cannot lock zoom bits");
		SetCursor(OldC);
		return;
	}	

	// Set Pointer arrays for access
	for(cnt = 0; cnt < YSIZE; cnt++)
	{
        	Array[cnt]    = (double *)(&PArray   [(long)cnt*lXSIZE]);
		LArray[cnt]   = (long *)  (&PLArray  [(long)cnt*lXSIZE]);
                OrgBits[cnt]  = (char *)  (&POrgBits [(long)cnt*lXSIZE]);
		ZoomBits[cnt] = (char *)  (&PZoomBits[(long)cnt*lXSIZE]);
	}

	HFILE hf;

	if((hf = _lopen(AFileName, READ)) == HFILE_ERROR)
	{
		// This should never happen because the existance has been checked
                Error("Open file <%s> cannot be opened (has been checked before???", AFileName);
		SetCursor(OldC);
		return;
	}

        SetCursor(OldC);
	OldC = SetCursor(LoadCursor(GetApplication()->hInstance, "CURSOR_LOAD"));

	_hread(hf, PArray, lXSIZE * lYSIZE * sizeof(double));

	_lread(hf, &CenterX, sizeof(CenterX));
	_lread(hf, &CenterY, sizeof(CenterY));
	_lread(hf, &Mini, sizeof(Mini));	// Mini and Maxi are reset in SetupWindow()
	_lread(hf, &Maxi, sizeof(Maxi));
	_lread(hf, &CenterH, sizeof(CenterH));
        // Only read size extensions if present
	if(_lread(hf, &Width, sizeof(Width)) < sizeof(Width))
	{
		Width  = dXS;
                Height = dYS;
	}
	else
        {
		_lread(hf, &Height, sizeof(Height));
	}
        _lclose(hf);

	DestroyCursor(SetCursor(OldC));
        OldC = SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Copy values to long array
	lCenterX = CenterX;
	lCenterY = CenterY;
	lCenterH = d2l(CenterH);
	lWidth   = Width;
        lHeight  = Height; 

	// Allocate bitmap headers and set color indices
	if((CSBitmapInfo = (BMI *)malloc(sizeof(BMI))) == NULL)
	{
		Error("Cannot create ColorScale Bitmapheader");
		SetCursor(OldC);
                return;
	}
	CSBitmapInfo->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
	CSBitmapInfo->bmiHeader.biWidth		= 4;
	CSBitmapInfo->bmiHeader.biHeight	= MAXCOLORS;
	CSBitmapInfo->bmiHeader.biPlanes	= 1;
	CSBitmapInfo->bmiHeader.biBitCount	= 8;
	CSBitmapInfo->bmiHeader.biCompression	= BI_RGB;
	CSBitmapInfo->bmiHeader.biSizeImage	= CSBitmapInfo->bmiHeader.biWidth * CSBitmapInfo->bmiHeader.biHeight;
	CSBitmapInfo->bmiHeader.biXPelsPerMeter	= 0;
	CSBitmapInfo->bmiHeader.biYPelsPerMeter	= 0;
	CSBitmapInfo->bmiHeader.biClrUsed	= 0;
	CSBitmapInfo->bmiHeader.biClrImportant	= 0;

	for(cnt = 0; cnt < 256; cnt++)
	{
		((int *)&(CSBitmapInfo->bmiColors))[cnt] = cnt;
	}
										
	if((BitmapInfo = (BMI *)malloc(sizeof(BMI))) == NULL)
	{
		Error("Cannot create org Bitmapheader");
		SetCursor(OldC);
                return;
	}
	BitmapInfo->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
	BitmapInfo->bmiHeader.biWidth		= lXSIZE;
	BitmapInfo->bmiHeader.biHeight		= lYSIZE;
	BitmapInfo->bmiHeader.biPlanes		= 1;
	BitmapInfo->bmiHeader.biBitCount	= 8;
	BitmapInfo->bmiHeader.biCompression	= BI_RGB;
	BitmapInfo->bmiHeader.biSizeImage	= BitmapInfo->bmiHeader.biWidth * BitmapInfo->bmiHeader.biHeight;
	BitmapInfo->bmiHeader.biXPelsPerMeter	= 0;
	BitmapInfo->bmiHeader.biYPelsPerMeter	= 0;
	BitmapInfo->bmiHeader.biClrUsed		= 0;
	BitmapInfo->bmiHeader.biClrImportant	= 0;

	for(cnt = 0; cnt < 256; cnt++)
	{
		((int *)&(BitmapInfo->bmiColors))[cnt] = cnt;
	}

	if((ZBitmapInfo = (BMI *)malloc(sizeof(BMI))) == NULL)
	{
		Error("Cannot create zoomed Bitmapheader");
		SetCursor(OldC);
                return;
	}
	ZBitmapInfo->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
	ZBitmapInfo->bmiHeader.biWidth		= lXSIZE;
	ZBitmapInfo->bmiHeader.biHeight		= lYSIZE;
	ZBitmapInfo->bmiHeader.biPlanes		= 1;
	ZBitmapInfo->bmiHeader.biBitCount	= 8;
	ZBitmapInfo->bmiHeader.biCompression	= BI_RGB;
	ZBitmapInfo->bmiHeader.biSizeImage	= ZBitmapInfo->bmiHeader.biWidth * ZBitmapInfo->bmiHeader.biHeight;
	ZBitmapInfo->bmiHeader.biXPelsPerMeter	= 0;
	ZBitmapInfo->bmiHeader.biYPelsPerMeter	= 0;
	ZBitmapInfo->bmiHeader.biClrUsed	= 0;
	ZBitmapInfo->bmiHeader.biClrImportant	= 0;

	for(cnt = 0; cnt < 256; cnt++)
	{
		((int *)&(ZBitmapInfo->bmiColors))[cnt] = cnt;
	}									

        // Fill the ColorScale bitmap bits
	for(cnt = 10; cnt < MAXCOLORS+10; cnt++)
	{
		CScaleBits[cnt-10] = (long)cnt + ((long)cnt<<8) + ((long)cnt<<16) + ((long)cnt<<24);
	}

        // Allocate a logical palette
	if((LogPalette = (LOGPALETTE *)malloc(sizeof(LOGPALETTE) + 256*sizeof(PALETTEENTRY))) == NULL)
	{
        	Error("Cannot create LogPalette");
		SetCursor(OldC);
                return;
	}
	LogPalette->palVersion = 0x300;
	LogPalette->palNumEntries = 256;
	HDC DC = GetDC(NULL);
	GetSystemPaletteEntries(DC, 0, 10, &LogPalette->palPalEntry[0]);
	GetSystemPaletteEntries(DC, 246, 10, &LogPalette->palPalEntry[246]);
	ReleaseDC(NULL, DC);

	// Get path of last selected palette and load if possible
        char path[MAXPATH];
	if(GetIniParameter(eip_LastPaletteLoaded, path, sizeof(path)))
	{
		PaletteFile = new TPaletteFile(path);
		if(PaletteFile != NULL && PaletteFile->Initialized)
		{ 
			memcpy(&LogPalette->palPalEntry[10], PaletteFile->Palette, MAXCOLORS*sizeof(PALETTEENTRY));
		}
		else if(PaletteFile != NULL)
		{
			delete PaletteFile;
			PaletteFile = NULL;
                        goto DefaultPalette;
		}
		else
		{
                	goto DefaultPalette;
		} 
	}
	else
	{
DefaultPalette:
		// If palettefile cannot be loaded, create default
		for(cnt = 10; cnt < 246; cnt++)
		{
			LogPalette->palPalEntry[cnt].peRed   = (BYTE)(127.5 * (1.0 + sin((double)(cnt-10)/20.0 * M_PI + 1.333*M_PI)));
			LogPalette->palPalEntry[cnt].peGreen = (BYTE)(127.5 * (1.0 + sin((double)(cnt-10)/30.0 * M_PI + 0.667*M_PI)));
			LogPalette->palPalEntry[cnt].peBlue  = (BYTE)(127.5 * (1.0 + sin((double)(cnt-10)/70.0 * M_PI)));
			LogPalette->palPalEntry[cnt].peFlags = PC_RESERVED;
		}
	}

	if((hPalette = CreatePalette(LogPalette)) == NULL)
	{
               	Error("Cannot create Palette");
		SetCursor(OldC);
                return;
	}

	// Install the speed-buttons
	SButton[0]  = new TBButton(this, IDSB_PRINT,       "", SBUTX(0),  SBUTY(0),  SBUTW, SBUTH, FALSE);
	SButton[1]  = new TBButton(this, IDSB_PRINTSETUP,  "", SBUTX(1),  SBUTY(1),  SBUTW, SBUTH, FALSE);
	SButton[2]  = new TBButton(this, IDSB_ZOOMIN,      "", SBUTX(2),  SBUTY(2),  SBUTW, SBUTH, FALSE);
	SButton[3]  = new TBButton(this, IDSB_ZOOMOUT,     "", SBUTX(3),  SBUTY(3),  SBUTW, SBUTH, FALSE);
	SButton[4]  = new TBButton(this, IDSB_CROSS,       "", SBUTX(4),  SBUTY(4),  SBUTW, SBUTH, FALSE);
	SButton[5]  = new TBButton(this, IDSB_MAJORCROSS,  "", SBUTX(5),  SBUTY(5),  SBUTW, SBUTH, FALSE);
	SButton[6]  = new TBButton(this, IDSB_PALETTE,     "", SBUTX(6),  SBUTY(6),  SBUTW, SBUTH, FALSE);
	SButton[7]  = new TBButton(this, IDSB_SETCENTER,   "", SBUTX(7),  SBUTY(7),  SBUTW, SBUTH, FALSE);
	SButton[8]  = new TBButton(this, IDSB_PARAMETER,   "", SBUTX(8),  SBUTY(8),  SBUTW, SBUTH, FALSE);
	SButton[9]  = new TBButton(this, IDSB_PREFERENCES, "", SBUTX(9),  SBUTY(9),  SBUTW, SBUTH, FALSE);
	SButton[10] = new TBButton(this, IDSB_SCALETICKS,  "", SBUTX(10), SBUTY(10), SBUTW, SBUTH, FALSE);

	// Its done...
	SetCursor(OldC);

        Initialized = TRUE;	// ok to continue.
}


//
//*****************************************************************************
// Function	: TDumpWindow::~TDumpWindow
// Syntax	: ~TDumpWindow()
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
TDumpWindow::~TDumpWindow()
{
	if(Array != NULL)	free(Array);
	if(PArray != NULL)	GlobalUnlock(hArray);
	if(hArray != NULL)	GlobalFree(hArray);
	if(LArray != NULL)	free(LArray);
	if(PLArray != NULL)	GlobalUnlock(hLArray);
	if(hLArray != NULL)	GlobalFree(hLArray);
        if(BitmapInfo != NULL)	free(BitmapInfo);
	if(ZBitmapInfo != NULL)	free(ZBitmapInfo);
	if(CSBitmapInfo != NULL)free(CSBitmapInfo);
        if(hPalette != NULL)	DeleteObject(hPalette);
	if(LogPalette != NULL)	free(LogPalette);
        if(PaletteFile != NULL)	delete PaletteFile;
	if(OrgBits != NULL)	free(OrgBits);
	if(POrgBits != NULL)	GlobalUnlock(hOrgBits);
	if(hOrgBits != NULL)	GlobalFree(hOrgBits);
	if(ZoomBits != NULL)	free(ZoomBits);
	if(PZoomBits != NULL)	GlobalUnlock(hZoomBits);
	if(hZoomBits != NULL)	GlobalFree(hZoomBits);
	if(hFont != NULL)	DeleteObject(hFont);
        if(CursorZoom != NULL)	DestroyCursor(CursorZoom);
	if(CursorSetCenter != NULL)	DestroyCursor(CursorSetCenter);
}


//
//*****************************************************************************
// Function	: TDumpWindow::SetupWindow
// Syntax	: void SetupWindow(void)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TDumpWindow::SetupWindow(void)
{
	TWindow::SetupWindow();

	if(!Initialized)
	{
		CloseWindow();		// Close this window if not initialized
                return;
	}

	int x, y;
	double *cacheptr;
	long *lcacheptr;
        char *ccacheptr;
	double mi = HUGE_VAL;
        double ma = -HUGE_VAL;

	HCURSOR OldC = SetCursor(LoadCursor(GetApplication()->hInstance, "CURSOR_CALC"));

	// Determine minimum, maximum and copy to long array
	for(y = 0; y < YSIZE; y++)
	{
		cacheptr  = Array[y];
                lcacheptr = LArray[y];
		for(x = 0; x < XSIZE; x++)
		{
			if(*cacheptr < INVALIDPOINTCOMP)
                        {
				if(*cacheptr < mi) mi = *cacheptr;
				if(*cacheptr > ma) ma = *cacheptr;
				*lcacheptr = d2l(*cacheptr);
			}
			else
			{
                        	*lcacheptr = 0x7fffffffl;	
			}
			cacheptr++;
			lcacheptr++; 
		}
	}

	lMin = ZoomMin = d2l(Mini = mi);
	lMax = ZoomMax = d2l(Maxi = ma);

        // Fill the org bitmap bits
	long zmin = lMin;
	long zdif = lMax - lMin;

	for(y = 0; y < YSIZE; y++)
	{
		lcacheptr = LArray[y];
                ccacheptr = OrgBits[y];
		for(x = 0; x < XSIZE; x++)
		{
			if(*lcacheptr != 0x7fffffffl)
			{
				*(unsigned char *)ccacheptr = (unsigned char)((*lcacheptr - zmin) * (long)(MAXCOLORS-1) / zdif + 10);
			}
			else
			{
                        	*ccacheptr = 0;
			}
			lcacheptr++;
			ccacheptr++; 
		}
	}


	// This Show() things is to capture the first input from the MDI frame.
	// It seems that this window gets input after a z-order change
	// or something in like that... Don't know what is really the problem.
	// This trick (minimize then restore) seems to work (Hell Windows!).
        // BTW, How do you maximize a MDI-frame????
	Show(SW_SHOWMINIMIZED);	
	Show(SW_SHOWNORMAL);

	HDC DC = GetDC(HWindow);
	HPALETTE opal = SelectPalette(DC, hPalette, FALSE);
	RealizePalette(DC);
        SelectPalette(DC, opal, FALSE);
	ReleaseDC(HWindow, DC);

	CalcBorders();

	DestroyCursor(SetCursor(OldC));
}


//
//*****************************************************************************
// Function	:
// Syntax	: 
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TDumpWindow::CloseWindow(void)
{
}


//
//*****************************************************************************
// Function	: 
// Syntax	:
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TDumpWindow::WMQueryNewPalette(RTMessage Msg)
{
	HDC DC = GetDC(HWindow);
	HPALETTE opal = SelectPalette(DC, hPalette, FALSE);
	int c = RealizePalette(DC);
	SelectPalette(DC, opal, FALSE);
	ReleaseDC(HWindow, DC);
	if(c != 0)
	{
		InvalidateRect(HWindow, &ClientRect, TRUE);
                Msg.Result = TRUE;
	}
	else
	{
		Msg.Result = FALSE;
        }
}


