#ifndef __DUMPWIN_H
#define __DUMPWIN_H

#ifndef __WINDOW_H
#include <window.h>
#endif

#ifndef __BBUTTON_H
#include <bbutton.h>
#endif

#ifndef __VIEWDMPR_H
#include "viewdmpr.h"
#endif

#ifndef __PALFILE_H
#include "palfile.h"
#endif


#define CRX1		(ClientRect.left)	// ClientRect shortcuts
#define CRX2		(ClientRect.right)
#define CRY1		(ClientRect.top)
#define CRY2		(ClientRect.bottom)
#define CRW		(CRX2 - CRX1 + 1)
#define CRH		(CRY2 - CRY1 + 1)
#define CRWUSE		(CRW - LEFTMARGIN - RIGHTMARGIN)
#define CRHUSE		(CRH - TOPMARGIN - BOTTOMMARGIN)

#define LEFTMARGIN	((CRW * 16) / 100)	// Left   margin at 16% of window border
#define RIGHTMARGIN	((CRW * 25) / 100)	// Right  margin at 25% of window border
#define TOPMARGIN	((CRH * 15) / 100)	// Top    margin at 15% of window border
#define BOTTOMMARGIN	((CRH * 15) / 100)	// Bottom margin at 15% of window border
#define MAJORTICK	(0.040)			// Major tick size 4% of smallest aspected size (W or H)
#define MINORTICK	(0.020)			// Minor tick size 2% of smallest aspected size (W or H)
#define PALMARGIN	((CRW * 12) / 100)	// Palette box right margin 12%
#define PALWIDTH	((CRW *  3) / 100)	// Palette box width 3% of window width

#define PALX1		(PALX2-PALWIDTH)	// ColorScale display box
#define PALX2		(CRX2-PALMARGIN)
#define PALY1		(PRY1)
#define PALY2		(PRY2)
#define PALW		(PALX2 - PALX1 + 1)
#define PALH		(PALY2 - PALY1 + 1)

#define PRX1		(PicRect.left)		// Image display box = Picture box shortcuts
#define PRX2		(PicRect.right)
#define PRY1		(PicRect.top)
#define PRY2		(PicRect.bottom)
#define PRW		(PRX2 - PRX1 + 1)
#define PRH		(PRY2 - PRY1 + 1)

#define IW		(XSIZE)			// (i-coor) Image width
#define IH		(YSIZE)			// (i-coor) Image height
#define IVX1		(ZoomL)			// (i-coor) Image view left
#define IVX2		(ZoomR)			// (i-coor) Image view right
#define IVY1		(ZoomT)			// (i-coor) Image view top
#define IVY2		(ZoomB)			// (i-coor) Image view bottom
#define IVW		(ZoomW())		// (i-coor) Image view width
#define IVH		(ZoomH())		// (i-coor) Image view height


#define TOOLBAR		26
#define STATUSBAR	20
#define FONTFACTOR	2.5


#define MAXSBUT		15
#define SBUTW		26
#define SBUTH		26
#define SBUTX(x)	(SBUTW*(x)-2)
#define SBUTY(y)	(-2)


class TPaletteFile;	// Forward reference


enum eActionState
{
	eAct_Nop = 0,		// Idle action
	eAct_ZoomWait,		// Pushed button or menu to zoom
	eAct_ZoomSize,		// LButton down
	eAct_ZoomDrag,		// LButton + RButton down
	eAct_SetCenter,		// Set center at LButton down pos
	eAct_Cross,		// Take cross section 
	eAct_CrossDrag		// Drag Cross section center
};


// This is just a copy of the BITMAPINFO structure exept for
// the bmiColors var. It has been extended from 1 to 256
struct BMI
{
	BITMAPINFOHEADER	bmiHeader;
        RGBQUAD			bmiColors[256];
};

_CLASSDEF(TDumpWindow)

class TDumpWindow : public TWindow
{
public:
	BOOL		Initialized;	// TRUE if all memory is allocated properly, etc.

	// Org file info	 
	HGLOBAL		hArray;		// Main data array handle
	double huge 	*PArray;	// Locked pointer to main data array
	double		**Array;	// Locked pointer array access

	int		CenterX;	// Org center X [pix]
	int		CenterY;	// Org center Y [pix]
	double		Mini;		// Array minimum [mm]
	double		Maxi;		// Array maximum [mm]
	double		CenterH;	// Org center height [mm]
	double		Width;		// Org total width [mm]  
	double		Height;		// Org total height [mm]

	// Converted to long, file info
	HGLOBAL		hLArray;	// Converted data array handle
	long huge   	*PLArray;	// Locked pointer to converted data array
	long		**LArray;	// Locked pointer array access
	int		lCenterX;
        int		lCenterY;
	long		lMin;		// Long array min/max
	long		lMax;
	long		lCenterH;	// Converted center height
	double		lWidth;		// Used total width   
	double		lHeight;	// Used total height

	// Color variables
	TPaletteFile	*PaletteFile;	// The current loaded palette
	LOGPALETTE	*LogPalette;	// Logical Pallete
	HPALETTE	hPalette;	// Realized palette

        // Bitmap variables
        BMI		*BitmapInfo;	// Org bitmap info
	HGLOBAL		hOrgBits;	// Org bitmapbits handle
	char huge	*POrgBits;	// Org bitmapbits locked pointer
	char		**OrgBits;	// Org bitmapbits access 	

	BMI		*ZBitmapInfo;	// Zoomed bitmap info
	HGLOBAL		hZoomBits;	// Zoomed bitmapbits handle
	char huge	*PZoomBits;	// Zoomed bitmapbits Locked pointer
	char		**ZoomBits;	// Zoomed bitmapbits access

	BMI		*CSBitmapInfo;	// Colorscale bitmap info
	long		CScaleBits[MAXCOLORS];	// Colorscale BitmapBits;

	// Zoom variables
	BOOL		IsZoomedFlag;	// True if picture displayed zoomed

	int		ZoomWXStart;	// Start coordinates of zoom (during eAct_Zoom...)
	int		ZoomWYStart;
	int		ZoomWXBox;	// Box (at cursor) size of zoom (during eAct_Zoom...)
	int		ZoomWYBox;

	long		ZoomMin;	// Minimum and maximum of zoomed area
	long		ZoomMax;

	int		ZoomT;		// Zoomed area Top, Bottom, Left, Right with respect to Array
	int		ZoomB;
	int		ZoomL;
	int		ZoomR;

	// Displayed cursor during several ActionStates
	HCURSOR		Cursor;		// Displayed cursor in window
	HCURSOR		CursorZoom;	// Preloaded zoom cursor
	HCURSOR		CursorSetCenter; // Preloaded set center cursor

	int		csidx;		// Color scale index line y pos

        // Generic vars
	int		PointSize;	// Font Pointsize
	HFONT		hFont;		// Font to draw in

	eActionState	ActionState;	// State of action during zoom etc.

	RECT		ClientRect;	// Current boxes position. Note that these rectangles
	RECT		PicRect;	//  have inclusive significant right and bottom members

	int		Wcx;		// Picture box Center pos
	int		Wcy;

	int		LoAngle;	// Lo visible angle scale
	int		HiAngle;	// Hi visible angle scale

	int		MajorTick;	// Angle scale tick sizes calculated from outer clip
        int		MinorTick;

	PTBButton	SButton[MAXSBUT];	// Pointers to the speed-buttons

public:
	TDumpWindow(PTWindowsObject AParent, LPSTR ATitle, LPSTR AFileName, PTModule AModule = NULL);
	~TDumpWindow();

	virtual LPSTR GetClassName()
		{ return("TDumpWindow"); };
	virtual void GetWindowClass(WNDCLASS _FAR &aWndClass)
		{ TWindow::GetWindowClass(aWndClass);
		  aWndClass.hCursor = NULL;
		  aWndClass.hIcon = LoadIcon(GetApplication()->hInstance, "ICON_VIEWDUMP");
		};

	virtual void SetupWindow(void);
	virtual void CloseWindow(void);
	virtual void Paint( HDC PaintDC, PAINTSTRUCT& RPS);

	virtual void WMLButtonDown(RTMessage Msg)	= [WM_FIRST + WM_LBUTTONDOWN];
	virtual void WMLButtonUp(RTMessage Msg)		= [WM_FIRST + WM_LBUTTONUP];
	virtual void WMRButtonDown(RTMessage Msg)	= [WM_FIRST + WM_RBUTTONDOWN];
	virtual void WMRButtonUp(RTMessage Msg)		= [WM_FIRST + WM_RBUTTONUP];
	virtual void WMMouseMove(RTMessage Msg)		= [WM_FIRST + WM_MOUSEMOVE];
	virtual void WMQueryNewPalette(RTMessage Msg)	= [WM_FIRST + WM_QUERYNEWPALETTE];
	virtual void WMSize(RTMessage Msg)		= [WM_FIRST + WM_SIZE];
        virtual void WMSetFocus(RTMessage Msg)		= [WM_FIRST + WM_SETFOCUS];
	virtual void WMKillFocus(RTMessage Msg)		= [WM_FIRST + WM_KILLFOCUS];

	BOOL XYInRect(int x, int y, RECT& r);
	int  Bound(int var, int mini, int maxi);
	void Swap(int& a, int& b);
	void InvertZoomBox(HDC DC);
	int  IVX2WX(int x);
	int  IVY2WY(int y);
	int  WX2IVX(int x);
	int  WY2IVY(int y);

	long d2l(double val)		{ return (long)(val * 1e6 + 0.5); };
	double l2d(long val)		{ return (double)val / 1e6; };
	BOOL ArrayIsZoomed(void)	{ return IsZoomedFlag; };
        int  ZoomW(void)		{ return ZoomR - ZoomL + 1; };
	int  ZoomH(void)		{ return ZoomB - ZoomT + 1; };

	void PaintDegreeScale(HDC DC);
	void PaintPictureScale(HDC PaintDC);
	void PaintColorScale(HDC PaintDC);

	void CalcBorders(void);
	double CalcTick(double dif);

	BOOL Clipt(double denom, double num, double& tE, double &tL);
	BOOL Clip2D(double& x0, double& y0, double& x1, double& y1, RECT& cliprect);

	void Error(char *fmt, ...);
	int Question(char *fmt, ...);


	virtual void CMFilePrint(RTMessage Msg)		= [CM_FIRST + CM_FILEPRINT];
	virtual void CMFilePrintSetup(RTMessage Msg)	= [CM_FIRST + CM_FILEPRINTSETUP];

	virtual void CMViewZoomIn(RTMessage Msg)	= [CM_FIRST + CM_VIEWZOOMIN];
	virtual void CMViewZoomOut(RTMessage Msg)	= [CM_FIRST + CM_VIEWZOOMOUT];
	virtual void CMViewCross(RTMessage Msg)		= [CM_FIRST + CM_VIEWCROSS];
	virtual void CMViewMajorCross(RTMessage Msg)	= [CM_FIRST + CM_VIEWMAJORCROSS];
	virtual void CMViewPalette(RTMessage Msg)	= [CM_FIRST + CM_VIEWPALETTE];
	virtual void CMViewSetCenter(RTMessage Msg)	= [CM_FIRST + CM_VIEWSETCENTER];
	virtual void CMViewParameter(RTMessage Msg)	= [CM_FIRST + CM_VIEWPARAMETER];
	virtual void CMViewPreferences(RTMessage Msg)	= [CM_FIRST + CM_VIEWPREFERENCES];
	virtual void CMViewScaleTicks(RTMessage Msg)	= [CM_FIRST + CM_VIEWSCALETICKS];

	virtual void BMPrint(RTMessage Msg)		= [ID_FIRST + IDSB_PRINT];
	virtual void BMPrintSetup(RTMessage Msg)	= [ID_FIRST + IDSB_PRINTSETUP];
	virtual void BMZoomIn(RTMessage Msg)		= [ID_FIRST + IDSB_ZOOMIN];
	virtual void BMZoomOut(RTMessage Msg)		= [ID_FIRST + IDSB_ZOOMOUT];
	virtual void BMCross(RTMessage Msg)		= [ID_FIRST + IDSB_CROSS];
	virtual void BMMajorCross(RTMessage Msg)	= [ID_FIRST + IDSB_MAJORCROSS];
	virtual void BMPalette(RTMessage Msg)		= [ID_FIRST + IDSB_PALETTE];
	virtual void BMSetCenter(RTMessage Msg)		= [ID_FIRST + IDSB_SETCENTER];
	virtual void BMParameter(RTMessage Msg)		= [ID_FIRST + IDSB_PARAMETER];
	virtual void BMPreferences(RTMessage Msg)	= [ID_FIRST + IDSB_PREFERENCES];
	virtual void BMScaleTicks(RTMessage Msg)	= [ID_FIRST + IDSB_SCALETICKS];
};

inline BOOL TDumpWindow::XYInRect(int x, int y, RECT& r)
{
	return	x >= r.left  &&
		x <= r.right &&
		y >= r.top   &&
		y <= r.bottom;
}

inline int TDumpWindow::Bound(int var, int mini, int maxi)
{
	if(var < mini) var = mini;
	if(var > maxi) var = maxi;
        return var;
}

inline void TDumpWindow::Swap(int& a, int& b)
{
	int tmp = a;
	a = b;
        b = tmp;
}

// Image view i-coor to w-coor picturebox relative
inline int TDumpWindow::IVX2WX(int x)
{
	return (int)(((long)(x-ZoomL) * (long)PRW) / (long)ZoomW()) + PRX1 + PRW/(2*ZoomW());
}

inline int TDumpWindow::IVY2WY(int y)
{
	return PRH - (int)(((long)(y-ZoomT) * (long)PRH) / (long)ZoomH()) + PRY1 - PRH/(2*ZoomH());
}

// Window picturebox relative w-coor to image view i-coor
inline int TDumpWindow::WX2IVX(int x)
{
	return (int)(((long)(x-PRX1) * (long)ZoomW()) / (long)PRW) + ZoomL;
}

inline int TDumpWindow::WY2IVY(int y)
{
	return (int)(((long)(PRH-(y-PRY1)) * (long)ZoomH()) / (long)PRH) + ZoomT;
}


#endif
