//
//************************************************************************
// File                 : dumpwinp.cpp  
// Module               : 	
// Project              : View Dumpfile  
// Description          : dumpwin.cpp support Painting
// Creation date        : 
// Last update          : 
// Author(s)            : Bertho Stultiens (BS)
// Version              :
// Compiler             : Borland C++ 3.1
//	switches	:
// Remarks              :
// Modification History :
//************************************************************************
// 

#include <owl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <bwcc.h>
#include "dlgparm.h"
#include "viewdmp.h"
#include "dumpwin.h"


//
//*****************************************************************************
// Function	: 
// Syntax	: 
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TDumpWindow::Paint(HDC PaintDC, PAINTSTRUCT&)
{
	HCURSOR OldC = SetCursor(LoadCursor(NULL, IDC_WAIT));		// Set Wait cursor
	HPALETTE oldpal = SelectPalette(PaintDC, hPalette, FALSE);	// Select our palette
	HFONT HOldFont = (HFONT)SelectObject(PaintDC, hFont);		// Select correct font
        int bltmode = SetStretchBltMode(PaintDC, STRETCH_DELETESCANS);	// Eliminate non fitting pixels

	// Paint default boxes and lines
	MoveTo(PaintDC, CRX1, CRY1);				// Line under toolbar
	LineTo(PaintDC, CRX2, CRY1);
	MoveTo(PaintDC, CRX1, CRY2);				// Line over statusbar
	LineTo(PaintDC, CRX2, CRY2);
        Rectangle(PaintDC, PRX1-1, PRY1-1, PRX2+2, PRY2+2);	// Box surrounding(!) picture 
	Rectangle(PaintDC, PALX1-1, PALY1-1, PALX2+2, PALY2+2);	// Box surrounding(!) colorscale

        // Paint scales
	PaintDegreeScale(PaintDC);	// The surrounding degree scale
	PaintColorScale(PaintDC);	// Paint colorscale markers and text
	PaintPictureScale(PaintDC);	// Paint picturescale markers and text

	// Paint the ColorScale
	StretchDIBits(PaintDC, PALX1, PALY1, PALW, PALH, 0, 0, 4, 236, CScaleBits, (BITMAPINFO *)CSBitmapInfo, DIB_PAL_COLORS, SRCCOPY);

	// Paint the contents
	if(ArrayIsZoomed())
	{
		StretchDIBits(PaintDC, PRX1, PRY1, PRW, PRH, 0, 0, ZoomW(), ZoomH(), PZoomBits, (BITMAPINFO *)ZBitmapInfo, DIB_PAL_COLORS, SRCCOPY);
	}
	else
        {
		StretchDIBits(PaintDC, PRX1, PRY1, PRW, PRH, 0, 0, XSIZE, YSIZE, POrgBits, (BITMAPINFO *)BitmapInfo, DIB_PAL_COLORS, SRCCOPY);
        }

	// Cleanup some things
        SelectPalette(PaintDC, oldpal, FALSE);
	SelectObject(PaintDC, HOldFont);
        SetCursor(OldC);
	SetStretchBltMode(PaintDC, bltmode);
}


//
//*****************************************************************************
// Function	: TDumpWindow::CalcBorders
// Syntax	: void CalcBorders(void)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
inline int dxdy2ang(int x0, int y0, int x1, int y1)
{
	double dx = x1 - x0;
	double dy = y1 - y1;

	if(dx == 0.0)
	{
        	return 90;
	}
	else
	{
        	return (int)(atan2(dy, dx) / M_PI * 180.0 + 180.5);
        }
}


void TDumpWindow::CalcBorders(void)
{
	double aspectxy = lWidth / lHeight;		// Image x/y aspect ratio

	// Setup paintable rectangle
	GetClientRect(HWindow, &ClientRect);
	CRY1 += TOOLBAR;
        CRY2 -= STATUSBAR;

	// Calculate picture coordinates in correct xy aspect that will fit in window
	PRX1 = CRX1 + LEFTMARGIN;	// First guess without aspect ratio
        PRX2 = CRX2 - RIGHTMARGIN;
	PRY1 = CRY1 + TOPMARGIN;
        PRY2 = CRY2 - BOTTOMMARGIN;

	if((double)PRH * aspectxy > (double)PRW)	// Check for fit with aspect 
	{
		int prh = PRH - (int)((double)PRW / aspectxy);
		PRY1 += prh / 2;
                PRY2 -= prh / 2;
	}
	else
	{
		int prw = PRW - (int)((double)PRH * aspectxy);
		PRX1 += prw / 2;
                PRX2 -= prw / 2;
	}

	// Calculate imagecentre in window coordinates
	Wcx = IVX2WX(lCenterX);	       
        Wcy = IVY2WY(lCenterY);

	// Calculate degreescale ticklength and fontsize
	if((double)CRWUSE > (double)CRHUSE * aspectxy)
	{
		MajorTick = (int)(MAJORTICK * (double)CRHUSE);
		MinorTick = (int)(MINORTICK * (double)CRHUSE);
	}
	else
	{
		MajorTick = (int)(MAJORTICK * (double)CRWUSE);
		MinorTick = (int)(MINORTICK * (double)CRWUSE);
	}

	// Prebuild font for drawing
	PointSize = (int)(sqrt(CRHUSE+CRWUSE) / FONTFACTOR);
	if(PointSize < 4) PointSize = 4;
	if(hFont != NULL) DeleteObject(hFont);	// Remove old font 
	hFont = CreateFont(-PointSize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, PROOF_QUALITY, (VARIABLE_PITCH | 0x04 | FF_SWISS), "Helv");

	// Determine degreescale visible angles
	if(XYInRect(Wcx, Wcy, PicRect))
	{
		LoAngle = 0;	// If center within displayed area -> all angles
		HiAngle = 359; 
	}
	else
	{
        	// Center not in displayed box => find minimum and maximum angles
        	int alt = dxdy2ang(Wcx, Wcy, PRX1, PRY1);
		int alb = dxdy2ang(Wcx, Wcy, PRX1, PRY2);
		int art = dxdy2ang(Wcx, Wcy, PRX2, PRY1);
		int arb = dxdy2ang(Wcx, Wcy, PRX2, PRY2);

		int LoAngle = alt;
		if(alb < LoAngle) LoAngle = alb; 
		if(art < LoAngle) LoAngle = art;
		if(arb < LoAngle) LoAngle = arb;
		int HiAngle = alt;
		if(alb > HiAngle) HiAngle = alb; 
		if(art > HiAngle) HiAngle = art;
		if(arb > HiAngle) HiAngle = arb;

		LoAngle -= LoAngle % 5;		// Round to first 5 deg boundary
                HiAngle += 5 - (HiAngle % 5); 
	}
}


//
//*****************************************************************************
// Function	: TDumpWindow::PaintDegreeScale
// Syntax	: void PaintDegreeScale(HDC DC)
// Input	:
//	DC	- The Paint display context
// Output	: nop 
// Description	: Put degree markers at the ClientRect outer clip rect with length
//		  defined with MajorTick and MinorTick generating inner clipping
//		  rectangles. 
// Remarks	: We could use Windows clipping but this has the side effect that
//		  it is unknown where the beginning of a marker is. And we want to
//		  know for the text positioning.
//		  Because the window origin is upper left corner, the angles have been
//		  converted.
//*****************************************************************************
//
void TDumpWindow::PaintDegreeScale(HDC DC)
{
	RECT majortr = {CRX1+MajorTick, CRY1+MajorTick, CRX2-MajorTick, CRY2-MajorTick};
	RECT minortr = {CRX1+MinorTick, CRY1+MinorTick, CRX2-MinorTick, CRY2-MinorTick};

	for(int angle = LoAngle; angle <= HiAngle; angle += 5)
	{
		double dcx = (double)Wcx;
                double dcy = (double)Wcy;
		double x = (double)(CRW + CRH) * cos((double)angle / 180.0 * M_PI) + dcx;
		double y = (double)(CRW + CRH) * sin((double)angle / 180.0 * M_PI) + dcy;
		double tcx = x;
                double tcy = y;

		if((angle % 15) == 0)
		{
			Clip2D(dcx, dcy, x, y, majortr);	// Clip inner space to x, y
		}
		else
		{ 
			Clip2D(dcx, dcy, x, y, minortr);	// Clip inner space to x, y
		}
		Clip2D(dcx, dcy, tcx, tcy, ClientRect);		// Clip outer space to tcx, tcy
		 
		MoveTo(DC, (int)(tcx+0.5), (int)(tcy+0.5));	// Draw tick line
		LineTo(DC, (int)(x+0.5), (int)(y+0.5));

		if((angle % 15) == 0)
                {
			char str[6];
			int cnt = sprintf(str, "%d", (360 - angle) % 360);
			DWORD size = GetTextExtent(DC, str, cnt);
			int tw = LOWORD(size);
			int th = HIWORD(size);
			int tx;
			int ty;

			if(angle < 22 || angle >= 337)		// Right
			{
				tx = (int)x - tw;
				ty = (int)y - th/2; 
			}
			else if(angle >= 22 && angle < 67)	// Right bottom
			{
				tx = (int)x - tw;
                                ty = (int)y - th;
			}
			else if(angle >= 67 && angle < 112)	// Bottom
			{
				tx = (int)x - tw/2;
                                ty = (int)y - th;
			}
			else if(angle >= 112 && angle < 157)	// Bottom left
			{
				tx = (int)x;
                                ty = (int)y - th;
			}
			else if(angle >= 157 && angle < 202)	// Left
			{
				tx = (int)x;
                                ty = (int)y - th/2;
			}
			else if(angle >= 202 && angle < 247)	// Left Top
			{
				tx = (int)x;
                                ty = (int)y;
			}
			else if(angle >= 247 && angle < 292)	// Top
			{
				tx = (int)x - tw/2;
                                ty = (int)y;
			}
			else if(angle >= 292 && angle < 337)	// Top right
			{
				tx = (int)x - tw;
                                ty = (int)y;
			}

			TextOut(DC, tx, ty, str, cnt);
		}

	}
}


//
//*****************************************************************************
// Function	: TDumpWindow::CalcTick
// Syntax	: double CalcTick(double dif)
// Input	:
// Output	: 
// Description	: 
// Remarks	: This could be done quicker with bi-section or so... But who cares.
//		  BTW Divides can cause major roundoff errors and therefore
//		  are unusable
//*****************************************************************************
//
double TDumpWindow::CalcTick(double dif)
{
	if(dif >= 5000.0)	return 5000.0;
	else if(dif >= 2000.0)	return 2000.0;
	else if(dif >= 1000.0)	return 1000.0;
	else if(dif >= 500.0)	return 500.0;
	else if(dif >= 200.0)	return 200.0;
	else if(dif >= 100.0)	return 100.0;
	else if(dif >= 50.0)	return 50.0;
	else if(dif >= 20.0)	return 20.0;
	else if(dif >= 10.0)	return 10.0;
	else if(dif >= 5.0)	return 5.0;
	else if(dif >= 2.0)	return 2.0;
	else if(dif >= 1.0)	return 1.0;
	else if(dif >= 0.5)	return 0.5;
	else if(dif >= 0.2)	return 0.2;
	else if(dif >= 0.1)	return 0.1;
	else if(dif >= 0.05)	return 0.05;
	else if(dif >= 0.02)	return 0.02;
	else if(dif >= 0.01)	return 0.01;
	else if(dif >= 0.005)	return 0.005;
	else if(dif >= 0.002)	return 0.002;
	else if(dif >= 0.001)	return 0.001;
	else if(dif >= 0.0005)	return 0.0005;
	else if(dif >= 0.0002)	return 0.0002;
	else			return 0.0001;	// 0.1um
}


//
//*****************************************************************************
// Function	: 
// Syntax	: double TickStart(double start, double ticksize)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
inline double TickStart(double start, double ticksize)
{
	double base = fmod(start, ticksize);	
	if(base == 0.0)
	{
        	base = start; 
	}
	else if(start < 0.0)
	{
		base = start - base;
	}
	else
	{
        	base = start - base + ticksize; 
	}
        return base;
}


//
//*****************************************************************************
// Function	: TDumpWindow::PaintColorScale
// Syntax	: void PaintColorScale(HDC PaintDC)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
#define CSMAJOR		5
#define CSMINOR		3
		 	
void TDumpWindow::PaintColorScale(HDC PaintDC)
{
	double mini = l2d(ZoomMin);		// Get actual min and max
	double maxi = l2d(ZoomMax);
	double zdif = maxi - mini;

	// Check text size to calculate number of ticks possible without text overlaps
	DWORD ts = GetTextExtent(PaintDC, "1", 1);			// Size of text	
	double tick = CalcTick(zdif/(double)(PALH/(HIWORD(ts)*4)));	// Value of markers
	double base = TickStart(mini, tick);				// Calculate the value of the first marker

	int acc = ceil(fabs(log10(tick)));	// Get the same number of decimals for all markers
	if(acc == 0) acc = 1;			// At least display one decimal

	for(; base <= maxi; base += tick)
	{
		int y = PALY2 - PALH/(2*MAXCOLORS) - (int)((double)PALH * (base-mini) / zdif);
		MoveTo(PaintDC, PALX2+2, y);
		LineTo(PaintDC, PALX2+2+CSMAJOR, y);
		char str[16];
                int cnt;
		if(tick >= 1.0)
                {
			cnt = sprintf(str, "%.1lf", base);
		}
                else
		{
			cnt = sprintf(str, "%1.*lf", acc, base);
		}
		ts = GetTextExtent(PaintDC, str, cnt);
		TextOut(PaintDC, PALX2+2+CSMAJOR+1, y-HIWORD(ts)/2, str, cnt);
		// Print a minor tick also
		if(base+tick/2.0 <= maxi)
		{
			y = PALY2 - PALH/(2*MAXCOLORS) - (int)((double)PALH * (base+tick/2.0-mini) / zdif);
			MoveTo(PaintDC, PALX2+2, y);
			LineTo(PaintDC, PALX2+2+CSMINOR, y);
		}
	}
}


//
//*****************************************************************************
// Function	: TDumpWindow::PaintPictureScale
// Syntax	: void PaintPictureScale(HDC PaintDC)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
#define PRXMAJOR	6
#define PRXMINOR	3
#define PRYMAJOR	8
#define PRYMINOR	4

void TDumpWindow::PaintPictureScale(HDC PaintDC)
{
	double xs = lWidth * (double)(ZoomW()) / dXSIZE;	// Visible x range
	double ys = lHeight * (double)(ZoomH()) / dYSIZE;	// Visible y range
	double minx = (double)(ZoomL-lCenterX)/dXSIZE*lWidth;
	double miny = (double)(ZoomT-lCenterY)/dXSIZE*lHeight;
	double maxx = (double)(ZoomR-lCenterX)/dXSIZE*lWidth;
	double maxy = (double)(ZoomB-lCenterY)/dXSIZE*lHeight;

	// Check text size to calculate number of ticks possible
	DWORD ts = GetTextExtent(PaintDC, "0.05", 4);			// Size of text
	double ticky = CalcTick(ys/(double)(PRH/(HIWORD(ts)*4)));	// Value of markers
	double tickx = CalcTick(xs/(double)(PRW/(LOWORD(ts)*3)));
	double basey = TickStart(miny, ticky);
	double basex = TickStart(minx, tickx);

	int accx = ceil(fabs(log10(tickx)));
	int accy = ceil(fabs(log10(ticky)));

	if(accx == 0) accx = 1;		// Display one decimal minimal
	if(accy == 0) accy = 1;

	for(; basex <= maxx; basex += tickx)
	{
		int x = PRX1 + PRW/(2*ZoomW()) + (int)((double)PRW * (basex-minx) / xs);
		MoveTo(PaintDC, x, PRY1-2);
		LineTo(PaintDC, x, PRY1-2-PRXMAJOR);
		MoveTo(PaintDC, x, PRY2+2);
		LineTo(PaintDC, x, PRY2+2+PRXMAJOR);
		char str[16];
                int cnt;
		if(tickx >= 1.0)
                {
			cnt = sprintf(str, "%.1lf", basex);
		}
                else
		{
			cnt = sprintf(str, "%1.*lf", accx, basex);
		}
		ts = GetTextExtent(PaintDC, str, cnt);
                TextOut(PaintDC, x-LOWORD(ts)/2, PRY2+2+PRXMAJOR, str, cnt);
		// Print a minor tick also
		if(basex+tickx/2.0 <= maxx)
		{
			x = PRX1 + PRW/(2*ZoomW()) + (int)((double)PRW * (basex+tickx/2.0-minx) / xs);
			MoveTo(PaintDC, x, PRY1-2);
			LineTo(PaintDC, x, PRY1-2-PRXMINOR);
			MoveTo(PaintDC, x, PRY2+2);
			LineTo(PaintDC, x, PRY2+2+PRXMINOR);
		}
	}

	for(; basey <= maxy; basey += ticky)
	{
		int y = PRY2 - PRH/(2*ZoomH()) - (int)((double)PRH * (basey-miny) / ys);
		MoveTo(PaintDC, PRX1-2, y);
		LineTo(PaintDC, PRX1-2-PRYMAJOR, y);
		MoveTo(PaintDC, PRX2+2, y);
		LineTo(PaintDC, PRX2+2+PRYMAJOR, y);
		char str[16];
                int cnt;
		if(ticky >= 1.0)
                {
			cnt = sprintf(str, "%.1lf", basey);
		}
                else
		{
			cnt = sprintf(str, "%1.*lf", accy, basey);
		}
		ts = GetTextExtent(PaintDC, str, cnt);
		TextOut(PaintDC, PRX1-2-LOWORD(ts)-PRYMAJOR, y-HIWORD(ts)/2 , str, cnt);
		// Print a minor tick also
		if(basey+ticky/2.0 <= maxy)
		{
			y = PRY2 - PRH/(2*ZoomH()) - (int)((double)PRH * (basey+ticky/2.0-miny) / ys);
			MoveTo(PaintDC, PRX1-2, y);
			LineTo(PaintDC, PRX1-2-PRYMINOR, y);
			MoveTo(PaintDC, PRX2+2, y);
			LineTo(PaintDC, PRX2+2+PRYMINOR, y);
		}
	}
}

