//
//************************************************************************
// File                 : dumpwins.cpp  
// Module               : 	
// Project              : View Dumpfile  
// Description          : dumpwin.cpp support methods generic
// Creation date        : 16-Apr-1995 
// Last update          : 
// Author(s)            : Bertho Stultiens (BS)
// Version              :
// Compiler             : Borland C++ 3.1
//	switches	:
// Remarks              :
// Modification History :
//************************************************************************
// 

#include <owl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <bwcc.h>
#include "dlgparm.h"
#include "viewdmp.h"
#include "dumpwin.h"


//
//*****************************************************************************
// Function	: TDumpWindow::Error
// Syntax	: void Error(char *fmt, ...)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TDumpWindow::Error(char *fmt, ...)
{
	char str[256];
	va_list va;

	va_start(va, fmt);
	vsprintf(str, fmt, va);
	BWCCMessageBox(HWindow, str, "Error...", MB_OK | MB_ICONHAND);
	va_end(va);
}


//
//*****************************************************************************
// Function	: TDumpWindow::Question
// Syntax	: int Question(char *fmt, ...)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
int TDumpWindow::Question(char *fmt, ...)
{
	char str[256];
	va_list va;
	int retval;

	va_start(va, fmt);
	vsprintf(str, fmt, va);
	retval = BWCCMessageBox(HWindow, str, "Question...", MB_YESNO | MB_ICONQUESTION);
	va_end(va);
        return retval;
}


//
//*****************************************************************************
// Function	: TDumpWindow::Clipt
// Syntax	: BOOL Clipt(double denom, double num, double& tE, double &tL)
// Input	:
// Output	: 
// Description	:  
// Remarks	: Helper routine for Clip2D()
//*****************************************************************************
//
BOOL TDumpWindow::Clipt(double denom, double num, double& tE, double& tL)
{
	double t;
	BOOL accept = TRUE;

	if(denom > 0.0)
	{
		t = num / denom;
		if(t > tL)
		{
                	accept = FALSE;
		}
		else if(t > tE)
		{
                	tE = t;
                }
	}
	else if(denom < 0.0)
	{
		t = num / denom;
		if(t < tE)
		{
                	accept = FALSE;
		}
		else if(t < tL)
		{
                	tL = t;
                }
	}
	else if(num > 0.0)
	{
        	accept = FALSE;
	}
        return accept;
}


//
//*****************************************************************************
// Function	: TDumpWindow::Clip2D
// Syntax	: BOOL Clip2D(double& x0, double& y0, double& x1, double& y1, RECT& cliprect)
// Input	:
// Output	: TRUE if part or whole line is visible, FALSE if line outside
//		  clipping rectangle. 
// Description	: This routine is a copy from "Computer Graphics", J. Foley, A. van Dam,
//		  S. Feiner, J. Hughes; Adisson Wesley, pp 117-124. 
// Remarks	: This routine is based on  Y.D. Liang and B.A. Barsky,
//		  "A new concept and method for line clipping", ACM TOG, 3(1),
//		  January 1984, 1-22.
//		  This routine is accompanied with helper routine Clipt()
//*****************************************************************************
//
BOOL TDumpWindow::Clip2D(double& x0, double& y0, double& x1, double& y1, RECT& cliprect)
{
	double tE = 0.0;
	double tL = 1.0;
	double dx = x1 - x0;
	double dy = y1 - y0;

	if(dx == 0.0 && dy == 0.0 &&
	   x0 > cliprect.left &&
	   x0 < cliprect.right &&
	   y0 > cliprect.top &&
	   y0 < cliprect.bottom)
	{
        	return TRUE;
	}

	if(Clipt(dx, (double)cliprect.left - x0, tE, tL))
	{
		if(Clipt(-dx, x0 - (double)cliprect.right, tE, tL))
		{
			if(Clipt(dy, (double)cliprect.top - y0, tE, tL))
			{
				if(Clipt(-dy, y0 - (double)cliprect.bottom, tE, tL))
				{
					if(tL < 1.0)
					{
						x1 = x0 + tL * dx;
                                                y1 = y0 + tL * dy;
					}
					if(tE > 0.0)
					{
						x0 = x0 + tE * dx;
                                                x0 = x0 + tE * dy;
					}
					return TRUE;					        	
			        }
			}
	        }
	}

        return FALSE;
}


