/*
 ******************************************************************************
 * File                 : ini.cpp
 * Module               : 
 * Project              : View dumpfile
 * Description          : Ini file management
 * Creation date        : 31-Mrt-1993
 * Last update          : 23-Apr-1995
 * Author(s)            : Bertho Stultiens (BS)
 * Version              : 
 * Compiler             : Borland C++ 3.1
 *      switches        :
 * Remarks              :
 * Modification History :
 * BS   31-Mrt-1993:    Created this file
 * BS	23-Apr-1995:	Copied from EKM project and changed the group array
 *			to work for this project.
 ******************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "ini.h"

static char EmptyString[] = "";		// Just what it says

union ParmTypes			// Collection of different types used
{
	char		Char;
	unsigned char	UChar;
	int		Int;
	unsigned int	UInt;
	long		Long;
	unsigned long	ULong;
        float		Float;
	double		Double;
	char 	       *String;
        EOnOff		OnOff;
};

enum EIniParameterType		// Type enumeration
{
	eipt_Char,
        eipt_UChar,
	eipt_Int,
	eipt_UInt,
	eipt_Long,
	eipt_ULong,
        eipt_Float,
	eipt_Double,	
	eipt_String,
	eipt_OnOff,

	eipt_None	// last one    
};

// The entries of all sections with corresponding strings
enum EIniParameterEntry
{
	eipe_PaletteDir = 0,
	eipe_LastPaletteLoaded,

        eipe_LastEntry
};

char *IniParameterEntry[] =		// Must be in the order of EIniParameterEntry
{
	"PaletteDir",
	"LastPaletteLoaded",

        EmptyString			// last entry
};

// The sections of the ini-file with corresponding strings
enum EIniParameterSection
{
	eips_Palette = 0,

        eips_LastSection
};

char *IniParameterSection[] =		// Must be in the order of EIniParameterSection
{
	"Palette",

        EmptyString		// last section
};

// The structure for definition of section, entry and type for all entries 
struct TIniParameterGroup
{
	EIniParameter		eip; 	// The parameter id
	EIniParameterEntry	eipe;	// has this entry name index,
	EIniParameterSection	eips;	// this section name index
        EIniParameterType	eipt;	// and this type
};

TIniParameterGroup IniParameterGroup[] =
{
	{eip_PaletteDir,	eipe_PaletteDir,	eips_Palette,		eipt_String},
	{eip_LastPaletteLoaded,	eipe_LastPaletteLoaded,	eips_Palette,		eipt_String},

        {eip_LastParameter,	eipe_LastEntry,		eips_LastSection,	eipt_None}
};

#define INIPARAMETERGROUPS	(sizeof(IniParameterGroup) / sizeof(TIniParameterGroup))

/*
 ******************************************************************************
 * Function     : FindIniParameter
 * Syntax       : int FindIniParameter(EIniParameter parm)
 * Input        :
 *	parm	- The ini parameter id
 * Output       : An index to the IniParameterGroup array or -1 of no entry
 *		  is found
 * Description  : Searches the IniParameterGroup array for the occurance of the
 *		  entry identified by 'parm'.
 * Remarks      :
 ******************************************************************************
 */
int FindIniParameter(EIniParameter parm)
{
	int cnt;

	for(cnt = 0; cnt < INIPARAMETERGROUPS; cnt++)
	{
		if(IniParameterGroup[cnt].eip == parm)
		{
                	return cnt;
                }
	}

        return -1;
}


/*
 ******************************************************************************
 * Function     : GetEntrySectionString
 * Syntax       : BOOL GetEntrySectionString(int index, char **entry, char **section)
 * Input        :
 *	index	- An index in the IniParameterGroup array
 *	entry	- A pointer to a string pointer to hold the entry name string address
 *	section	- A pointer to a string pointer to hold the section name string address
 * Output       : TRUE if both entry and section are valid, FALSE otherwise.
 * Description  : The IniParameterGroup array is indexed at 'index' to locate the
 *		  index for section and entry in the IniParameterEntry and
 *		  IniParameterSection arrays.
 * Remarks      :
 ******************************************************************************
 */
BOOL GetEntrySectionString(int index, char **entry, char **section)
{
	BOOL retval = TRUE;

	if(index >= INIPARAMETERGROUPS)		// illegal index
	{
		*entry = EmptyString;
		*section = EmptyString;
		return FALSE;	
	}

	if(IniParameterGroup[index].eipe < eipe_LastEntry)
	{
		*entry = IniParameterEntry[IniParameterGroup[index].eipe];
	}
	else
	{
		*entry = EmptyString;
                retval = FALSE;
        }

	if(IniParameterGroup[index].eips < eips_LastSection)
	{
		*section = IniParameterSection[IniParameterGroup[index].eips];
	}
	else
	{
		*section = EmptyString;
                retval = FALSE;
	}
        return retval;
}

 
/*
 ******************************************************************************
 * Function     : IniParameter2String
 * Syntax       : BOOL IniParameter2String(int index, char *str, ParmTypes *parm)
 * Input        :
 *	index	- An index to the IniParameterGroup array
 *	str	- A buffer to hold the converted input
 *	parm	- A pointer to a ParmTypes union which holds the data to convert
 * Output       : TRUE is conversion was succesful, FALSE otherwise. The converted
 *		  input is placed in 'str'.
 * Description  : IniParameter2String converts the 'parm' to a string according to
 *		  the type of the parameter. If an invalid index or type is
 *		  encountered, the returned string is an empty string;  
 * Remarks      :
 ******************************************************************************
 */
BOOL IniParameter2String(int index, char *str, ParmTypes *parm)
{
	if(index >= INIPARAMETERGROUPS)		// illegal index
	{
		str[0] = '\0';
		return FALSE;	
	}

	switch(IniParameterGroup[index].eipt)
	{
	case eipt_Char:
		sprintf(str, "%d", parm->Char);
		break;

	case eipt_UChar:
		sprintf(str, "0x%02X", parm->UChar);
		break;

	case eipt_Int:
		sprintf(str, "%d", parm->Int);
		break;

	case eipt_UInt:
		sprintf(str, "0x%04X", parm->UInt);
		break;

	case eipt_Long:
		sprintf(str, "%ld", parm->Long);
		break;

	case eipt_ULong:
		sprintf(str, "0x%08lX", parm->ULong);
		break;

	case eipt_Float:
		sprintf(str, "%.15g", parm->Float);
		break;

	case eipt_Double:
		sprintf(str, "%.15lg", parm->Double);
		break;

	case eipt_String:
		sprintf(str, "%s", parm->String);
		break;

	case eipt_OnOff:
		sprintf(str, "%s", parm->OnOff == eb_On ? "On" : "Off");
		break;

	default:
		str[0] = '\0';
                return FALSE;
	}
        return TRUE;
}


/*
 ******************************************************************************
 * Function     : IniString2Parameter
 * Syntax       : BOOL IniString2Parameter(int index, char *str, ParmTypes *parm, int maxlen)
 * Input        :
 *	index	- An index to the IniParameterGroup array
 *	str	- A buffer which holds the string to convert
 *	parm	- A pointer to a ParmTypes union which will hold the converted data
 *	maxlen	- Maximum length of string to copy if string type
 * Output       : TRUE is conversion was succesful, FALSE otherwise. The converted
 *		  input is placed in 'parm'.
 * Description  : IniString2Parameter converts the 'parm' to a value according to
 *		  the type of the parameter. If an invalid index or type is
 *		  encountered, the union value is set to 0.  
 * Remarks      :
 ******************************************************************************
 */
BOOL IniString2Parameter(int index, char *str, ParmTypes *parm, int maxlen)
{

	if(index >= INIPARAMETERGROUPS)		// illegal index
	{
		memset(parm, 0, sizeof(ParmTypes));    
		return FALSE;
	}

	switch(IniParameterGroup[index].eipt)
	{
	case eipt_Char:
		parm->Char = (char)strtol(str, NULL, 0);
		break;

	case eipt_UChar:
		parm->UChar = (unsigned char)strtoul(str, NULL, 0);
		break;

	case eipt_Int:
		parm->Int = (int)strtol(str, NULL, 0);
		break;

	case eipt_UInt:
		parm->UInt = (unsigned int)strtoul(str, NULL, 0);
		break;

	case eipt_Long:
		parm->Long = strtol(str, NULL, 0);
		break;

	case eipt_ULong:
		parm->ULong = strtoul(str, NULL, 0);
		break;

	case eipt_Float:
		parm->Float = (float)strtod(str, NULL);
		break;

	case eipt_Double:
		parm->Double = strtod(str, NULL);
		break;

	case eipt_String:
		strncpy(parm->String, str, maxlen);
		parm->String[maxlen-1] = '\0';		// To ensure termination
		break;

	case eipt_OnOff:
		if(toupper(str[0]) == 'O' && toupper(str[1]) == 'N')
		{
                	parm->OnOff = eb_On;
		}
		else
		{
			parm->OnOff = eb_Off;
		}
                break;

	default:
		memset(parm, 0, sizeof(ParmTypes));
		return FALSE;
	}
        return TRUE;
}


/*
 ******************************************************************************
 * Function     : SetIniParameterExt
 * Syntax       : BOOL SetIniParameterExt(EIniParameter parm, EIniParameterType type,
 *					  ParmTypes *value)
 * Input        :
 *	parm	- Parameter id to set
 *	type	- Parameter type id
 *	value	- A pointer to ParmTypes union which holds the data
 * Output       : TRUE if parameter is set correctly in the ini-file, FALSE in case of
 *		  an error which can be:
 *			- 'parm' is illegal
 *			- 'type' does not match expected type in IniParameterGroup array
 *			- Windows WritePrivateProfileString failed for some reason 
 * Description  : First an index is generated from 'parm'. Next is the expected type
 *		  verified with the actual datatype. The parameter is converted to a
 *		  string and the section and entry names are searched. Then the actual
 *		  profile string is written to the ini-file. 
 * Remarks      :
 ******************************************************************************
 */
BOOL SetIniParameterExt(EIniParameter parm, EIniParameterType type, ParmTypes *value)
{
	char buf[128];
	char *entry;
	char *section;
	int index;

        // Find the references for section, entry and type
	if((index = FindIniParameter(parm)) == -1)
	{
        	return FALSE;
	}

        // We can only use correct types here
	if(IniParameterGroup[index].eipt != type)
	{
        	return FALSE;
        }

	// Select the correct names
	if(!GetEntrySectionString(index, &entry, &section))
	{
        	return FALSE;
        }

        // Convert the input parameter to a string
	if(!IniParameter2String(index, buf, value))
	{
        	return FALSE;
        }

	// Write to the ini file
	if(WritePrivateProfileString(section, entry, buf, INIFILENAME) == 0)
	{
        	return FALSE;
	}
	else
	{
        	return TRUE;
        }
}


/*
 ******************************************************************************
 * Function     : SetIniParameter
 * Syntax       : BOOL SetIniParameter(EIniParameter parm, aType& value)
 * Input        :
 *	parm	- Parameter to set
 *	value	- The value to write with 'aType' one of the types defined in
 *		  the ParmTypes union. 
 * Output       : TRUE if value is written correctly, FALSE otherwise
 * Description  : Calls the generic conversion and profile write routine with
 *		  typed parameters.
 * Remarks      : 
 ******************************************************************************
 */
BOOL SetIniParameter(EIniParameter parm, char& value)
{
	return SetIniParameterExt(parm, eipt_Char, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, unsigned char& value)
{
	return SetIniParameterExt(parm, eipt_UChar, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, int& value)
{
	return SetIniParameterExt(parm, eipt_Int, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, unsigned int& value)
{
	return SetIniParameterExt(parm, eipt_UInt, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, long& value)
{
	return SetIniParameterExt(parm, eipt_Long, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, unsigned long& value)
{
	return SetIniParameterExt(parm, eipt_ULong, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, float& value)
{
	return SetIniParameterExt(parm, eipt_Float, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, double& value)
{
	return SetIniParameterExt(parm, eipt_Double, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, char *value)
{
	return SetIniParameterExt(parm, eipt_String, (ParmTypes *)&value);
}

BOOL SetIniParameter(EIniParameter parm, EOnOff& value)
{
	return SetIniParameterExt(parm, eipt_OnOff, (ParmTypes *)&value);
}


/*
 ******************************************************************************
 * Function     : GetIniParameterExt
 * Syntax       : BOOL GetIniParameterExt(EIniParameter parm, EIniParameterType type,
 *					  ParmTypes *value)
 * Input        :
 *	parm	- Parameter id to get
 *	type	- Parameter type id
 *	value	- A pointer to ParmTypes union which gets the data
 * Output       : TRUE if parameter is retrieved correctly in the ini-file, FALSE in
 *		  case of an error which can be:
 *			- 'parm' is illegal
 *			- 'type' does not match expected type in IniParameterGroup array
 *			- Windows GetPrivateProfileString failed for some reason 
 * Description  : First an index is generated from 'parm'. Next is the expected type
 *		  verified with the actual datatype. The section and entry names are
 *		  searched. Then the actual profile string is read and converted to
 *		  the correcty data format. 
 * Remarks      :
 ******************************************************************************
 */
BOOL GetIniParameterExt(EIniParameter parm, EIniParameterType type, ParmTypes *value, int maxlen)
{
	char buf[128];
	char *entry;
	char *section;
	int index;

        // Find the references for section, entry and type
	if((index = FindIniParameter(parm)) == -1)
	{
        	return FALSE;
	}

        // We can only use correct types here
	if(IniParameterGroup[index].eipt != type)
	{
        	return FALSE;
        }


	// Select the correct names
	if(!GetEntrySectionString(index, &entry, &section))
	{
        	return FALSE;
        }

	// Read the ini file
	if(GetPrivateProfileString(section, entry, EmptyString, buf, sizeof(buf), INIFILENAME) == 0)
	{
        	return FALSE;
	}

	// Convert the input parameter to a string
	if(!IniString2Parameter(index, buf, value, maxlen))
	{
        	return FALSE;
        }
	else
	{
        	return TRUE;
        }
}


/*
 ******************************************************************************
 * Function     : GetIniParameter
 * Syntax       : BOOL GetIniParameter(EIniParameter parm, aType& value)
 * Input        :
 *	parm	- Parameter to get
 *	value	- The value to read with 'aType' one of the types defined in
 *		  the ParmTypes union. 
 * Output       : TRUE if value is retrieved correctly, FALSE otherwise.
 * Description  : Calls the generic conversion and profile read routine with
 *		  typed parameters.
 * Remarks      : 
 ******************************************************************************
 */
BOOL GetIniParameter(EIniParameter parm, char& value)
{
	return GetIniParameterExt(parm, eipt_Char, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, unsigned char& value)
{
	return GetIniParameterExt(parm, eipt_UChar, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, int& value)
{
	return GetIniParameterExt(parm, eipt_Int, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, unsigned int& value)
{
	return GetIniParameterExt(parm, eipt_UInt, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, long& value)
{
	return GetIniParameterExt(parm, eipt_Long, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, unsigned long& value)
{
	return GetIniParameterExt(parm, eipt_ULong, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, float& value)
{
	return GetIniParameterExt(parm, eipt_Float, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, double& value)
{
	return GetIniParameterExt(parm, eipt_Double, (ParmTypes *)&value, 0);
}

BOOL GetIniParameter(EIniParameter parm, char *value, int maxlen)
{
	return GetIniParameterExt(parm, eipt_String, (ParmTypes *)&value, maxlen);
}

BOOL GetIniParameter(EIniParameter parm, EOnOff& value)
{
	return GetIniParameterExt(parm, eipt_OnOff, (ParmTypes *)&value, 0);
}


