//
//************************************************************************
// File                 : palfile.cpp  
// Module               : 	
// Project              : View dumpfile	  
// Description          :
// Creation date        : 25-Apr-1995 
// Last update          : 
// Author(s)            : Bertho Stultiens (BS)
// Version              :
// Compiler             : Borland C++ 3.1
//	switches	:
// Remarks              :
// Modification History :
//************************************************************************
// 
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <dir.h>
#include <bwcc.h>
#include "palfile.h"


//
//*****************************************************************************
// Function	: TPaletteFile::TPaletteFile
// Syntax	: TPaletteFile(char *path)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
char *getline(char *str, int n, FILE *fp)
{
	if(fgets(str, n, fp) == NULL)
	{
        	return NULL;
	}

        // Be sure to read newline
	if(str[strlen(str)-1] != '\n')
	{
		while(!feof(fp) && fgetc(fp) != '\n')
                	;
	}
	else
	{
        	str[strlen(str)-1] = '\0';	// strip \n
	}        	
	return str;
}

void striplead(char *str)
{
	char *cptr = str;

	while(strchr(" \n\t\v", *cptr) != NULL)
	{
        	cptr++;
	}
        strcpy(str, cptr);
}

TPaletteFile::TPaletteFile(char *path)
{
	FILE *fp;
        int cnt;
	char str[128];
	char name[64];
        int ncolor;


	// Initialize variables
	Initialized = FALSE;
	Path	= NULL;
	Name	= NULL;
	Palette	= NULL;
        Next    = NULL;

	// Copy palettefile path        	
	if((Path = (char *)malloc(strlen(path)+1)) == NULL)
	{
		Error("TPaletteFile: No memory for path");
                return;
	}
	strcpy(Path, path);

	if((fp = fopen(Path, "rt")) == NULL)
	{
		Error("TPaletteFile: Palettefile <%s> cannot be opened for read", Path);
                return;
        }

	// Read the name of the palette: NAME <name>
	if(getline(str, sizeof(str), fp) == NULL)
	{
		Error("TPaletteFile: Palettefile <%s> Invalid format", Path);
		fclose(fp);
                return;
	}
	striplead(str);	// Strip leading whitespace
	striplead(str+4);	// Skip "NAME"
        strcpy(name, str+4);

	// Copy palette name
	if((Name = (char *)malloc(strlen(name)+1)) == NULL)
	{
		Error("TPaletteFile: No memory for Name");
                fclose(fp);
                return;
	}
	strcpy(Name, name);

        // Read the number of colors: COLORS <n>
	if(getline(str, sizeof(str), fp) == NULL)
	{
		Error("TPaletteFile: Palettefile <%s> Invalid format", Path);
		fclose(fp);
                return;
	}
	striplead(str);	// Strip leading whitespace
	sscanf(str, "%*s %d", &ncolor);

	if(ncolor != MAXCOLORS)
	{
		Error("Need %d COLORS. Defined in file %d", MAXCOLORS, ncolor);
		fclose(fp);
		return;
        }

	getline(str, sizeof(str), fp);	// Skip COLORMAPRECORD line

	// Copy palette name
	if((Palette = (PALETTEENTRY *)malloc(sizeof(PALETTEENTRY)*MAXCOLORS)) == NULL)
	{
		Error("TPaletteFile: No memory for Palette entries");
		fclose(fp);
                return;
	}

        // Read the color RGB lines
	for(cnt = 0; cnt < ncolor; cnt++)
	{
		if(getline(str, sizeof(str), fp) == NULL)
		{
			Error("TPaletteFile: Palettefile <%s> Invalid format", Path);
			fclose(fp);
        	        return;
		}
		striplead(str);	// Strip leading whitespace
                int r, g, b;
		if(sscanf(str, "%d %d %d", &r, &g, &b) != 3)
		{
			Error("TPaletteFile: Palettefile <%s> Invalid RGB entry", Path);
			fclose(fp);
        	        return;
		}
                Palette[cnt].peRed   = r;
		Palette[cnt].peGreen = g;
		Palette[cnt].peBlue  = b;
		Palette[cnt].peFlags = PC_RESERVED;
	}
	fclose(fp);

        Initialized = TRUE;
}


//
//*****************************************************************************
// Function	: TPaletteFile::TPaletteFile
// Syntax	: TPaletteFile(PTPaletteFile src)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
TPaletteFile::TPaletteFile(PTPaletteFile src)
{
	Initialized = FALSE;
	Next = NULL;

	if((Path = (char *)malloc(strlen(src->Path)+1)) == NULL)
	{
		Error("TPaletteFile: No memory Path");
                return;
	}
        strcpy(Path, src->Path);

	if((Name = (char *)malloc(strlen(src->Name)+1)) == NULL)
	{
		Error("TPaletteFile: No memory for Name");
                return;
	}
        strcpy(Name, src->Name);

	if((Palette = (PALETTEENTRY *)malloc(sizeof(PALETTEENTRY)*MAXCOLORS)) == NULL)
	{
		Error("TPaletteFile: No memory for Palette entries");
		return;
	}
	memcpy(Palette, src->Palette, sizeof(PALETTEENTRY)*MAXCOLORS);
	Initialized = TRUE;
}


//
//*****************************************************************************
// Function	: TPaletteFile::~TPaletteFile
// Syntax	: ~TPaletteFile()
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
TPaletteFile::~TPaletteFile()
{
	if(Path != NULL) 	free(Path);
	if(Name != NULL) 	free(Name);
	if(Palette != NULL)	free(Palette);
}

//
//*****************************************************************************
// Function	: TPaletteFile::Error
// Syntax	: void Error(char *fmt, ...)
// Input	:
// Output	: 
// Description	: 
// Remarks	:
//*****************************************************************************
//
void TPaletteFile::Error(char *fmt, ...)
{
	char str[256];
	va_list va;

	va_start(va, fmt);
	vsprintf(str, fmt, va);
	BWCCMessageBox(NULL, str, "Error...", MB_OK | MB_ICONHAND);
	va_end(va);
}


